<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:ve="http://schemas.openxmlformats.org/markup-compatibility/2006"
	xmlns:o="urn:schemas-microsoft-com:office:office" xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships" xmlns:m="http://schemas.openxmlformats.org/officeDocument/2006/math"
	xmlns:v="urn:schemas-microsoft-com:vml" xmlns:wp="http://schemas.openxmlformats.org/drawingml/2006/wordprocessingDrawing" xmlns:w10="urn:schemas-microsoft-com:office:word" xmlns:w="http://schemas.openxmlformats.org/wordprocessingml/2006/main"
	xmlns:wne="http://schemas.microsoft.com/office/word/2006/wordml" xmlns:pic="http://schemas.openxmlformats.org/drawingml/2006/picture" xmlns:a="http://schemas.openxmlformats.org/drawingml/2006/main"
	xmlns:fn="http://www.pageseeder.com/function" xmlns:dec="java:java.net.URLDecoder" exclude-result-prefixes="#all">
  
  
      <!-- Title is imported as normal text, applying the default word heading styles -->
  <!--##title##-->
	<xsl:template match="title" mode="content">
		<w:p>
			<w:pPr>
				<xsl:call-template name="apply-style" />
			</w:pPr>
			<xsl:apply-templates mode="content" />
		</w:p>
	</xsl:template>
        
    <!-- Headings are imported as normal text, applying the default word heading styles -->
  <!--##heading##-->
	<xsl:template match="heading" mode="content">
		<xsl:param name="labels" tunnel="yes" />
        <!-- Handle headings. Style mapped from element name -->
<!--         <xsl:message><xsl:value-of select="$labels"/></xsl:message> -->
		<w:p>
			<w:pPr>
				<xsl:call-template name="apply-style" />
<!--         <xsl:message> -->
<!--           <xsl:call-template name="apply-style" /> -->
<!--            </xsl:message> -->
        <xsl:choose>
          <xsl:when test="fn:labels-keep-heading-with-next($labels,@level)">
            <w:rPr>
              <w:vanish/>
              <w:specVanish/>
            </w:rPr>
          </xsl:when>
          <xsl:when test="fn:default-keep-heading-with-next(@level)">
            <w:rPr>
              <w:vanish/>
              <w:specVanish/>
            </w:rPr>
          </xsl:when>
        </xsl:choose>
			</w:pPr>
			<xsl:if test="@prefix">
				<xsl:choose>
					<xsl:when test="fn:heading-prefix-select-for-document-label($labels,@level)">
						<xsl:sequence select="fn:heading-prefix-value-for-document-label($labels,@level,current())" />
					</xsl:when>
					<xsl:when test="fn:heading-prefix-select-for-default-document(@level)">
						<xsl:sequence select="fn:heading-prefix-value-for-default-document(@level,current())" />
					</xsl:when>
					<xsl:otherwise>
						<w:r>
							<w:t xml:space="preserve"><xsl:value-of select="@prefix" /> </w:t>
						</w:r>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:if>
			<xsl:if test="@numbered = 'true'">
				<xsl:choose>
					<xsl:when test="fn:heading-numbered-select-for-document-label($labels,@level)">

						<xsl:sequence select="fn:heading-numbered-value-for-document-label($labels,@level,current())" />
					</xsl:when>
					<xsl:when test="fn:heading-numbered-select-for-default-document(@level)">
						<xsl:sequence select="fn:heading-numbered-value-for-default-document(@level,current())" />
					</xsl:when>
					<xsl:otherwise>
						<w:r>
							<w:t xml:space="preserve"><xsl:value-of select="@prefix" /></w:t>
						</w:r>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:if>
			<xsl:apply-templates mode="content" />
		</w:p>
	</xsl:template>

       <!--  A block is imported as a normal paragraph -->
  <!--##paraLabel##-->
	<xsl:template match="block" mode="content">
		<xsl:param name="labels" tunnel="yes" />
      <!--
        if paraLabel contains only inline elements or text, create w:p here,
      -->
      <xsl:if test="@label = 'caption'">
<!--      <xsl:message>block</xsl:message> -->
     </xsl:if>
		<xsl:variable name="id" select="concat(@label, '-', generate-id())" />
		<xsl:choose>
        <!-- when containing other block elements, including mixed content -->
        <!-- will not create w:p here -->
			<xsl:when test="matches(@label,fn:block-ignore-labels-with-document-label($labels))">
<!-- 			 <xsl:message>1</xsl:message> -->
			</xsl:when>

			<xsl:when test="matches(@label,fn:default-block-ignore-labels())">
<!-- 			 <xsl:message>2</xsl:message> -->
			</xsl:when>

			<xsl:when test="fn:has-block-elements(.)='true'">
<!-- 			 <xsl:message>3</xsl:message> -->
				<xsl:apply-templates mode="content" />
			</xsl:when>
			<xsl:otherwise>
<!-- 			 <xsl:message>otherwise</xsl:message> -->
          <!-- when containing only inline ele                                                                                  ments or text()-->
				<w:p>
					<w:pPr>
					
						<xsl:call-template name="apply-style" />
            <!--##list##-->
            <!--##nlist##-->
            <!--##item##-->
            <xsl:choose>
              <xsl:when test="fn:labels-keep-block-with-next($labels,@label)">
                <w:rPr>
                  <w:vanish/>
                  <w:specVanish/>
                </w:rPr>
              </xsl:when>
              <xsl:when test="fn:default-keep-block-with-next(@label)">
                <w:rPr>
                  <w:vanish/>
                  <w:specVanish/>
                </w:rPr>
              </xsl:when>
            </xsl:choose> 
						<xsl:if test="ancestor::item">
							<xsl:choose>
								<xsl:when test="position()=1">
									<w:numPr>
										<w:ilvl w:val="{count(ancestor::list)+count(ancestor::nlist)-1}" />
										<w:numId w:val="{fn:get-numbering-id(current())}" />
									</w:numPr>
								</xsl:when>
								<xsl:otherwise>
									<w:ind w:left="{720*(count(ancestor::list)+count(ancestor::nlist))}" />
								</xsl:otherwise>
							</xsl:choose>
						</xsl:if>
					</w:pPr>
					<xsl:apply-templates mode="content" />
				</w:p>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

  <!-- para are treated normally -->
  <!--##para##-->
	<xsl:template match="para" mode="content">
		<xsl:param name="labels" tunnel="yes" />
<!--     <xsl:message><xsl:value-of select="@indent"/>:<xsl:value-of select="."/></xsl:message> -->
<!-- <xsl:message>para</xsl:message> -->
		<w:p>
			<w:pPr>
			  <xsl:choose>
          <xsl:when test="fn:labels-keep-para-with-next($labels,@indent)">
            <w:rPr>
              <w:vanish/>
              <w:specVanish/>
            </w:rPr>
          </xsl:when>
          <xsl:when test="fn:default-keep-para-with-next(@indent)">
            <w:rPr>
              <w:vanish/>
              <w:specVanish/>
            </w:rPr>
          </xsl:when>
          <xsl:otherwise>
            <!-- do nothing -->
          </xsl:otherwise>
        </xsl:choose> 
        <xsl:choose>
          <xsl:when test="ancestor::item">
            <xsl:variable name="role" select="ancestor::*[name() = 'list' or name() = 'nlist'][1]/@role"/>
            <xsl:variable name="level" select="count(ancestor::list)+count(ancestor::nlist)"/>
            <xsl:variable name="list-type" select="ancestor::*[name() = 'list' or name() = 'nlist'][1]/name()"/>
            <xsl:choose>
            <xsl:when test="position()=1">
              <xsl:choose>
	              <xsl:when test="ancestor::item/parent::*[@role]">
<!-- 	                <xsl:message>1</xsl:message> -->
	                <w:pStyle w:val="{ancestor::item/parent::*/@role}"/>
	              </xsl:when>
	              <xsl:when test="fn:list-wordstyle-for-document-label($labels,$role,$level,$list-type) != ''">
<!-- 	                <xsl:message>2</xsl:message> -->
	<!--                 <w:pStyle> -->
	<!--                 <xsl:attribute name="w:val"><xsl:value-of select="fn:list-wordstyle-for-document-label($labels,$role,$level,$list-type)"/></xsl:attribute> -->
	<!--                 </w:pStyle> -->
	                <xsl:call-template name="apply-style" />
	              </xsl:when>
	              <xsl:when test="fn:list-wordstyle-for-default-document($role,$level,$list-type) != ''">
<!-- 	              <xsl:message>3</xsl:message> -->
	<!--                 <w:pStyle> -->
	<!--                 <xsl:attribute name="w:val"><xsl:value-of select="fn:list-wordstyle-for-default-document($role,$level,$list-type)"/></xsl:attribute> -->
	<!--                 </w:pStyle> -->
	                <xsl:call-template name="apply-style" />
	              </xsl:when>
	              <xsl:otherwise>
<!-- 	                <xsl:message>4</xsl:message> -->
	                <xsl:call-template name="apply-style" />
	              </xsl:otherwise>
	            </xsl:choose>
              <xsl:variable name="max-num-id">
                <xsl:choose>
                  <xsl:when test="doc-available(concat($_dotxfolder,$numbering-template))">
                    <xsl:value-of select="max(document(concat($_dotxfolder,$numbering-template))/w:numbering/w:num/number(@w:numId))"/>
                  </xsl:when>
                  <xsl:otherwise><xsl:value-of select="0"/></xsl:otherwise>
                </xsl:choose>
              </xsl:variable>
<!--          <xsl:message>precedingsbling:<xsl:value-of select="count(preceding-sibling::item)"/></xsl:message>     -->
<!--           <xsl:if test="count(preceding-sibling::item) = 0"> -->
            <w:numPr>
              <xsl:variable name="level">
                <xsl:choose>
                  <xsl:when test="ancestor::item/parent::*[@role]">
                    <xsl:value-of select="fn:get-level-from-role(ancestor::item/parent::*/@role,.)"/>
<!--                     <xsl:message><xsl:value-of select="fn:get-level-from-role(parent::*/@role,.)"/></xsl:message> -->
                  </xsl:when>
                  <xsl:otherwise>
                    <xsl:value-of select="count(ancestor::list)+count(ancestor::nlist) - 1"/>
                  </xsl:otherwise>
                </xsl:choose>
              </xsl:variable>
              <w:ilvl w:val="{$level}" />
              <xsl:variable name="current-pstyle">
                <xsl:variable name="call-style">
                  <xsl:call-template name="apply-style" />
                </xsl:variable>
                <xsl:value-of select="$call-style//@w:val"/>
              </xsl:variable>
              <xsl:variable name="current-num-id">
                <xsl:choose>
                  <xsl:when test="ancestor::list and document(concat($_dotxfolder,$numbering-template))//w:abstractNum/w:lvl/w:pStyle/@w:val = $current-pstyle">
                    <xsl:variable name="current-numid" select="document(concat($_dotxfolder,$numbering-template))//w:abstractNum[w:lvl/w:pStyle/@w:val = $current-pstyle]/@w:abstractNumId"/>
                   
                    <xsl:value-of select="document(concat($_dotxfolder,$numbering-template))//w:num[w:abstractNumId/@w:val = $current-numid][1]/@w:numId"/>
                  </xsl:when>
                  <xsl:when test="parent::*[@role]">
                    <xsl:value-of select="$max-num-id + count(preceding::*[name()='nlist'][@start]) + 1"/>
                  </xsl:when>
                  <xsl:otherwise>
                    <xsl:value-of select="$max-num-id + count(ancestor::*[name()='nlist'][last()]/
                                          preceding::*[name()='nlist']
                                          [not(ancestor::list or ancestor::nlist)]) + 1"/>
                  </xsl:otherwise>
                </xsl:choose>
              </xsl:variable>
              <w:numId w:val="{$current-num-id}" />
            </w:numPr>
            
            </xsl:when>
            <xsl:otherwise>
<!--               <xsl:message>here</xsl:message> -->
              <xsl:variable name="list-level" select="count(ancestor::list)+count(ancestor::nlist) + 1"/>
<!--               <xsl:message><xsl:value-of select="$list-level"/></xsl:message> -->
              <xsl:choose>
		            <xsl:when test="fn:para-list-level-paragraph-for-document-label($labels,$list-level) != ''">
<!-- 		              <xsl:message>1</xsl:message> -->
                  <xsl:variable name="style-name" select="fn:para-list-level-paragraph-for-document-label($labels,$list-level)"/>
		              <w:pStyle w:val="{document(concat($_dotxfolder,$styles-template))//w:style[w:name/@w:val = $style-name]/@w:styleId}"/>
		            </xsl:when>
		            <xsl:when test="fn:para-list-level-paragraph-for-default-document($list-level) != ''">
<!-- 		              <xsl:message>2</xsl:message> -->
                  <xsl:variable name="style-name" select="fn:para-list-level-paragraph-for-default-document($list-level)"/>
                  <w:pStyle w:val="{document(concat($_dotxfolder,$styles-template))//w:style[w:name/@w:val = $style-name]/@w:styleId}"/>
                
		            </xsl:when>
		            <xsl:otherwise>
<!-- 		              <xsl:message>3</xsl:message> -->
		              <w:pStyle w:val="BodyText"/>
		            </xsl:otherwise>
		          </xsl:choose>
              <w:ind w:left="{720*(count(ancestor::list)+count(ancestor::nlist))}" />
            </xsl:otherwise>
          </xsl:choose>
          </xsl:when>
          <xsl:otherwise>
            <xsl:call-template name="apply-style" />
		        <xsl:if test="string(@indent) != '' and @indent castable as xs:integer">
		          <w:ind w:left="{720*number(@indent)}" />
		        </xsl:if>
          </xsl:otherwise>
        </xsl:choose>
				
          <!--##list##-->
          <!--##nlist##-->
          <!--##item##-->
				<xsl:if test="ancestor::item">
					
				</xsl:if>
				<xsl:if test="@prefix">
					<xsl:choose>
						<xsl:when test="fn:para-prefix-select-for-document-label($labels,@indent)">
							<xsl:sequence select="fn:para-prefix-value-for-document-label($labels,@indent,current())" />
						</xsl:when>
						<xsl:when test="fn:para-prefix-select-for-default-document(@indent)">
							<xsl:sequence select="fn:para-prefix-value-for-default-document(@indent,current())" />
						</xsl:when>
						<xsl:otherwise>
							<w:r>
								<w:t xml:space="preserve"><xsl:value-of select="@prefix" /> </w:t>
							</w:r>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:if>
				<xsl:if test="@numbered = 'true'">
					<xsl:choose>
						<xsl:when test="fn:para-numbered-select-for-document-label($labels,@indent)">
							<xsl:sequence select="fn:para-numbered-value-for-document-label($labels,@indent,current())" />
						</xsl:when>
						<xsl:when test="fn:para-numbered-select-for-default-document(@indent)">
							<xsl:sequence select="fn:para-numbered-value-for-default-document(@indent,current())" />
						</xsl:when>
<!-- 						<xsl:otherwise> -->
<!-- 							<w:r> -->
<!-- 								<w:t xml:space="preserve"><xsl:value-of select="@prefix" /> </w:t> -->
<!-- 							</w:r> -->
<!-- 						</xsl:otherwise> -->
					</xsl:choose>
				</xsl:if>
			</w:pPr>
			<xsl:apply-templates mode="content" />
		</w:p>
	</xsl:template>
  
  <!--  properties are currently imported as normal paragraphs -->
<!-- 	<xsl:template match="property" mode="content"> -->
<!-- 		<w:p> -->
<!-- 			<xsl:apply-templates mode="content" /> -->
<!-- 		</w:p> -->
<!-- 	</xsl:template> -->
  <!--  preformat elements are currently imported as normal paragraphs -->
	<xsl:template match="preformat" mode="content">
		<w:p>
		  <w:pPr>
		    <xsl:call-template name="apply-style" />
		  </w:pPr>
			<xsl:apply-templates mode="content" />
		</w:p>
	</xsl:template>

</xsl:stylesheet>