<?xml version="1.0" encoding="utf-8"?>

  <!--
    This stylesheet transform openXML into PS Format
  
    @author Hugo Inacio 
    @copyright Allette Systems Pty Ltd 
  -->

<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:ve="http://schemas.openxmlformats.org/markup-compatibility/2006" xmlns:o="urn:schemas-microsoft-com:office:office"
	xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships" xmlns:m="http://schemas.openxmlformats.org/officeDocument/2006/math" xmlns:v="urn:schemas-microsoft-com:vml"
	xmlns:wp="http://schemas.openxmlformats.org/drawingml/2006/wordprocessingDrawing" xmlns:w10="urn:schemas-microsoft-com:office:word" xmlns:w="http://schemas.openxmlformats.org/wordprocessingml/2006/main"
	xmlns:wne="http://schemas.microsoft.com/office/word/2006/wordml" xmlns:pic="http://schemas.openxmlformats.org/drawingml/2006/picture" xmlns:a="http://schemas.openxmlformats.org/drawingml/2006/main"
	xmlns:rs="http://schemas.openxmlformats.org/package/2006/relationships" xmlns:cp="http://schemas.openxmlformats.org/package/2006/metadata/core-properties" xmlns:dc="http://purl.org/dc/elements/1.1/"
	xmlns:dcterms="http://purl.org/dc/terms/" xmlns:ps="http://www.pageseeder.com/editing/2.0" xmlns:fn="http://www.pageseeder.com/function" xmlns:xs="http://www.w3.org/2001/XMLSchema"
	exclude-result-prefixes="#all">


<!-- 
Template to match all paragraphs and ,according to the configuration file:
1. Creates Headings
2. Creates Block labels
3. Creates para elements
4. Creates lists
 -->
	<xsl:template match="w:p" mode="content">

    <!-- variable used to check if there is a number or not in the beginning of the paragraph -->
		<xsl:variable name="full-text">
			<xsl:for-each select=".//w:r/*">
				<xsl:choose>
					<xsl:when test="current()/name() = 'w:br'">
						<xsl:text> </xsl:text>
					</xsl:when>
					<xsl:when test="current()/name() = 'w:tab'">
						<xsl:text> </xsl:text>
					</xsl:when>
					<xsl:when test="current()/name() = 'w:noBreakHyphen'">
						<xsl:text>-</xsl:text>
					</xsl:when>
					<xsl:when test="current()/name() = 'w:t'">
						<xsl:value-of select="." />
					</xsl:when>
				</xsl:choose>
			</xsl:for-each>
		</xsl:variable>

		<xsl:variable name="style-name">
			<xsl:value-of select="w:pPr/w:pStyle/@w:val" />
		</xsl:variable>

<!--     <xsl:message select="$style-name"/> -->
<!--     <xsl:message select="$full-text"/> -->
		<xsl:variable name="current-num-id">
			<xsl:value-of select="fn:get-numid-from-style(.)" />
		</xsl:variable>
    
    <xsl:variable name="has-numbering-format" select="fn:has-numbering-format($style-name,current())" as="xs:boolean"/>
   

		<xsl:variable name="new-element">
			<xsl:value-of select="fn:get-psml-element($style-name)" />
		</xsl:variable>
<!-- <xsl:message select="$new-element"/> -->

		<xsl:choose>
			<xsl:when test="matches($style-name, $ignore-paragraph-match-list)" />  
		  
      <!-- strip out empty headings -->
			<xsl:when test="matches($new-element, 'heading') and string-join((w:r|w:hyperlink)//text(), '') = ''" />
			
			<xsl:when test="matches($new-element, 'caption') 
			               and ( (following-sibling::*[1][name() = 'w:tbl'][not(w:tblPr/w:tblStyle/@w:val)] and fn:get-caption-table-value($style-name) = 'default')
			                      or  (following-sibling::*[1][name() = 'w:tbl'][w:tblPr/w:tblStyle/@w:val= fn:get-caption-table-value($style-name)] and fn:get-caption-table-value($style-name) != '' ))" >
			</xsl:when>
	
      <!-- when headings not in tables -->
      <!--##heading##-->
			<xsl:when test="matches($new-element, 'heading') and not(ancestor::w:tbl)">
<!--         <xsl:message select="$has-numbering-format"/> -->
<!--         <xsl:message select="$numbering-paragraphs-list"/> -->
        
        <xsl:call-template name="create-heading">
			    <xsl:with-param name="style-name" select="$style-name"/>
			    <xsl:with-param name="current-num-id" select="$current-num-id"/>
			    <xsl:with-param name="current" select="current()"/>
          <xsl:with-param name="full-text" select="$full-text"/>
          <xsl:with-param name="has-numbering-format" select="$has-numbering-format"/>
			  </xsl:call-template>
				
			</xsl:when>
			<!-- when the style is matched to be a block in the config file -->
			<xsl:when test="matches($new-element, 'block')">
				<xsl:element name="{$new-element}">
					<xsl:attribute name="label">
            <xsl:value-of select="fn:get-block-label-from-psml-element($style-name)" />
          </xsl:attribute>
					<xsl:apply-templates select="./*" mode="content">
						<xsl:with-param name="full-text" select="$full-text" />
					</xsl:apply-templates>
				</xsl:element>
			</xsl:when>

			<xsl:when test="matches($new-element, 'preformat')">
				<xsl:element name="{$new-element}">
					<xsl:apply-templates select="./*" mode="content">
						<xsl:with-param name="full-text" select="$full-text" />
					</xsl:apply-templates>
				</xsl:element>
			</xsl:when>
      
			<!-- when the style is matched to be a inline in the config file -->
			<xsl:when test="matches($new-element, 'inline')">
				<para>
					<xsl:element name="{$new-element}">
						<xsl:attribute name="label">
	            <xsl:value-of select="fn:get-inline-label-from-psml-element($style-name)" />
	          </xsl:attribute>
						<xsl:apply-templates select="./*" mode="content">
							<xsl:with-param name="full-text" select="$full-text" />
						</xsl:apply-templates>
					</xsl:element>
				</para>
			</xsl:when>

			<xsl:when test="matches($new-element, 'monospace')">
				<para>
					<xsl:element name="{$new-element}">
						<xsl:apply-templates select="./*" mode="content">
							<xsl:with-param name="full-text" select="$full-text" />
						</xsl:apply-templates>
					</xsl:element>
				</para>
			</xsl:when>
			<!-- if the element is set to create a para in the config file -->
			<xsl:when test="matches($new-element,'para')">
			
			   <xsl:call-template name="create-para">
          <xsl:with-param name="style-name" select="$style-name"/>
          <xsl:with-param name="current-num-id" select="$current-num-id"/>
          <xsl:with-param name="current" select="current()"/>
          <xsl:with-param name="full-text" select="$full-text"/>
          <xsl:with-param name="has-numbering-format" select="$has-numbering-format"/>
        </xsl:call-template>
<!-- 				<xsl:element name="{$new-element}"> -->
<!-- 					<xsl:if test="fn:get-para-indent-value($style-name) != ''"> -->
<!-- 						<xsl:attribute name="indent"><xsl:value-of select="fn:get-para-indent-value($style-name)" /></xsl:attribute> -->
<!-- 					</xsl:if> -->
<!-- 					<xsl:if test="$numbering-list-prefix-exists"> -->
<!-- 						<xsl:analyze-string regex="({$numbering-match-list-prefix})(.*)" select="$full-text"> -->
<!-- 							<xsl:matching-substring> -->
<!-- 								<xsl:attribute name="prefix"> -->
<!--                       <xsl:value-of select="regex-group(1)" /> -->
<!--                     </xsl:attribute> -->
<!-- 							</xsl:matching-substring> -->
<!-- 						</xsl:analyze-string> -->
<!-- 					</xsl:if> -->
<!-- 					<xsl:if test="$numbering-list-autonumbering-exists"> -->
<!-- 						<xsl:analyze-string regex="({$numbering-match-list-autonumbering})(.*)" select="$full-text"> -->
<!-- 							<xsl:matching-substring> -->
<!-- 								<xsl:attribute name="numbered"> -->
<!--                       <xsl:value-of select="'true'" /> -->
<!--                     </xsl:attribute> -->
<!-- 							</xsl:matching-substring> -->
<!-- 						</xsl:analyze-string> -->
<!-- 					</xsl:if> -->

<!-- 					<xsl:apply-templates select="./*" mode="content"> -->
<!-- 						<xsl:with-param name="full-text" select="$full-text" /> -->
<!-- 					</xsl:apply-templates> -->
<!-- 				</xsl:element> -->
			</xsl:when>
			<!-- when list -->
			<!--##list##-->

			<xsl:when test="matches($style-name,$numbering-paragraphs-list) and not(w:pPr/w:numPr/w:numId/@w:val = '0')">
				<xsl:variable name="current-num-id">
<!-- 				  <xsl:choose> -->
<!-- 				    <xsl:when test="w:pPr/w:numPr/w:numId/@w:val"> -->
<!-- 				      <xsl:message select="w:pPr/w:numPr/w:numId/@w:val"/> -->
<!-- 				      <xsl:value-of select="fn:get-abstract-num-id-from-num-id(w:pPr/w:numPr/w:numId/@w:val)"/> -->
<!-- 				    </xsl:when> -->
<!-- 				    <xsl:otherwise> -->
				      <xsl:value-of select="fn:get-abstract-num-id-from-element(.)" />
<!-- 				    </xsl:otherwise> -->
<!-- 				  </xsl:choose> -->
				</xsl:variable>
				
				<xsl:variable name="current-abstract-num-id">
              <xsl:value-of select="fn:get-abstract-num-id-from-element(.)" />
        </xsl:variable>

				<xsl:variable name="current-level">
					<xsl:value-of select="fn:get-level-from-element(.)" />
				</xsl:variable>

				<xsl:variable name="is-bullet" as="xs:boolean"
					select="if ($numbering-document/w:numbering/w:abstractNum[@w:abstractNumId=$current-abstract-num-id]/w:lvl[@w:ilvl=$current-level]/w:numFmt/@w:val='bullet') then true() else false()" />

				<xsl:variable name="previous-heading-levels">
					<xsl:value-of select="fn:get-preceding-heading-level-from-element(.)" />
				</xsl:variable>

				<xsl:choose>
					<xsl:when test="not($convert-to-numbered-paragraphs)">
						<xsl:choose>
        
<!-- 		           Numbered paragraph -->
							<xsl:when test="preceding::w:p[1][matches(w:pPr/w:pStyle/@w:val,$heading-paragraphs-list)]">

								<xsl:call-template name="list">
									<xsl:with-param name="abstract-id" select="$current-num-id" />
									<xsl:with-param name="level" select="$current-level" />
									<xsl:with-param name="id" select="@id" />
								</xsl:call-template>

							</xsl:when>
							<xsl:when
								test="preceding::w:p[1][matches(w:pPr/w:pStyle/@w:val,$numbering-paragraphs-list)][not(matches(w:pPr/w:pStyle/@w:val,$heading-paragraphs-list))][fn:get-abstract-num-id-from-element(.) =$current-num-id][parent::* = current()/parent::*]"></xsl:when>
							<xsl:otherwise>
								<xsl:call-template name="list">
									<xsl:with-param name="abstract-id" select="$current-num-id" />
									<xsl:with-param name="level" select="$current-level" />
									<xsl:with-param name="id" select="@id" />
								</xsl:call-template>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:when>
					<xsl:otherwise>
						<para>
<!-- 						  <xsl:message>1::<xsl:value-of  select="number($current-level)"/>-<xsl:value-of  select="number($current-num-id)"/></xsl:message> -->
							<xsl:attribute name="indent" select="number($current-level) - number(fn:get-preceding-heading-level-from-element(.)) + 1" />
							<xsl:choose>
								<xsl:when test="not($is-bullet) and fn:get-numbered-paragraph-value($current-level + 1)='numbering'">
									<xsl:attribute name="numbered" select="'true'" />
								</xsl:when>
								<xsl:when test="not($is-bullet) and fn:get-numbered-paragraph-value($current-level + 1)='prefix' and $list-paragraphs/w:p[@id = current()/@id]">
<!--                   <xsl:message>3::<xsl:value-of  select="$has-numbering-format"/></xsl:message> -->
									<xsl:if test="$has-numbering-format and fn:get-numbering-value-from-paragraph-style(.,$style-name) != ''">
										<xsl:attribute name="prefix" select="fn:get-numbering-value-from-paragraph-style(.,$style-name)" />
									</xsl:if>
								</xsl:when>
								<xsl:when test="matches(fn:get-numbered-paragraph-value($current-level + 1),'inline=[\w|-|_]+') and $list-paragraphs/w:p[@id = current()/@id]">
									<xsl:if test="$has-numbering-format and fn:get-numbering-value-from-paragraph-style(.,$style-name) != ''">
										<xsl:variable name="inline-label" select="substring-after(fn:get-numbered-paragraph-value($current-level + 1),'inline=')" />
										<inline label="{$inline-label}">
											<xsl:value-of select="fn:get-numbering-value-from-paragraph-style(.,$style-name)" />
											<xsl:text> </xsl:text>
										</inline>
									</xsl:if>
								</xsl:when>
								<xsl:when test="fn:get-numbered-paragraph-value($current-level + 1) = 'text'">
									<xsl:if test="$has-numbering-format">
										<xsl:value-of select="fn:get-numbering-value-from-paragraph-style(.,$style-name)" />
										<xsl:text> </xsl:text>
									</xsl:if>
								</xsl:when>
								<xsl:otherwise>
                  <!-- do nothing -->
								</xsl:otherwise>
							</xsl:choose>
							<xsl:apply-templates select="./*" mode="content">
								<xsl:with-param name="full-text" select="$full-text" />
							</xsl:apply-templates>
						</para>
					</xsl:otherwise>
				</xsl:choose>

			</xsl:when>
			<xsl:when test="w:pPr/w:numPr[w:ilvl][w:numId] and not(matches($style-name,$numbering-paragraphs-list)) and not(w:pPr/w:numPr/w:numId/@w:val = '0')">
				<xsl:variable name="listID" select="w:pPr/w:numPr/w:numId/@w:val" />
				<xsl:variable name="abstractNumId" select="fn:get-abstract-num-id-from-element(.)" />
				<xsl:variable name="current-level">
					<xsl:value-of select="fn:get-level-from-element(.)" />
				</xsl:variable>

				<xsl:variable name="is-bullet" as="xs:boolean"
					select="if ($numbering-document/w:numbering/w:abstractNum[@w:abstractNumId=$abstractNumId]/w:lvl[@w:ilvl=$current-level]/w:numFmt/@w:val='bullet') then true() else false()" />

				<xsl:choose>
					<xsl:when test="not($convert-to-numbered-paragraphs)">
						<xsl:choose>
							<xsl:when test="preceding-sibling::w:p[1][./w:pPr/w:numPr/w:numId/@w:val]" />
							<xsl:otherwise>

								<xsl:call-template name="list">
									<xsl:with-param name="abstract-id" select="$numbering-document/w:numbering/w:num[@w:numId=$listID]/w:abstractNumId/@w:val" />
									<xsl:with-param name="level" select="w:pPr/w:numPr/w:ilvl/@w:val" />
									<xsl:with-param name="id" select="@id" />
								</xsl:call-template>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:when>
					<xsl:otherwise>
						<para>
<!-- 						  <xsl:message>2::<xsl:value-of  select="number($current-level)"/>-<xsl:value-of  select="$abstractNumId"/></xsl:message> -->
							<xsl:attribute name="indent" select="number($current-level) - number(fn:get-preceding-heading-level-from-element(.)) + 1" />
							<xsl:choose>
								<xsl:when test="not($is-bullet) and fn:get-numbered-paragraph-value($current-level + 1)='numbering'">
									<xsl:attribute name="numbered" select="'true'" />
								</xsl:when>
								<xsl:when test="not($is-bullet) and fn:get-numbered-paragraph-value($current-level + 1)='prefix' and $list-paragraphs/w:p[@id = current()/@id]">
<!-- 									<xsl:message>4::<xsl:value-of  select="$has-numbering-format"/></xsl:message> -->
									<xsl:if test="$has-numbering-format and fn:get-numbering-value-from-paragraph-style(.,$style-name) != ''">
										<xsl:attribute name="prefix" select="fn:get-numbering-value-from-paragraph-style(.,$style-name)" />
									</xsl:if>
								</xsl:when>
								<xsl:when test="matches(fn:get-numbered-paragraph-value($current-level + 1),'inline=[\w|-|_]+') and $list-paragraphs/w:p[@id = current()/@id]">
									<xsl:if test="$has-numbering-format and fn:get-numbering-value-from-paragraph-style(.,$style-name) != ''">
										<xsl:variable name="inline-label" select="substring-after(fn:get-numbered-paragraph-value($current-level + 1),'inline=')" />
										<inline label="{$inline-label}">
											<xsl:value-of select="fn:get-numbering-value-from-paragraph-style(.,$style-name)" />
											<xsl:text> </xsl:text>
										</inline>
									</xsl:if>
								</xsl:when>
								<xsl:when test="fn:get-numbered-paragraph-value($current-level + 1) = 'text'">
									<xsl:if test="$has-numbering-format and fn:get-numbering-value-from-paragraph-style(.,$style-name) != ''">
										<xsl:value-of select="fn:get-numbering-value-from-paragraph-style(.,$style-name)" />
										<xsl:text> </xsl:text>
									</xsl:if>
								</xsl:when>
								<xsl:otherwise>
                  <!-- do nothing -->
								</xsl:otherwise>
							</xsl:choose>
							<xsl:apply-templates select="./*" mode="content">
								<xsl:with-param name="full-text" select="$full-text" />
							</xsl:apply-templates>
						</para>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:when>
			<xsl:otherwise>
				<xsl:choose>  
				  <!-- if the element is set to create a block in the config file -->
					<xsl:when test="$paragraph-styles = 'block' and $style-name !=''">
						<block label="{$style-name}">
							<xsl:apply-templates select="./*" mode="content">
								<xsl:with-param name="full-text" select="$full-text" />
							</xsl:apply-templates>
						</block>
					</xsl:when>
					<xsl:otherwise>
						<para>
							<xsl:if test="$numbering-list-prefix-exists">
<!-- 								<xsl:message select="$numbering-match-list-prefix" /> -->
								<xsl:analyze-string regex="({$numbering-match-list-prefix})(.*)" select="$full-text">
									<xsl:matching-substring>
										<xsl:attribute name="prefix">
		                      <xsl:value-of select="regex-group(1)" />
		                    </xsl:attribute>
									</xsl:matching-substring>
								</xsl:analyze-string>
							</xsl:if>
							<xsl:if test="$numbering-list-autonumbering-exists">
								<xsl:analyze-string regex="({$numbering-match-list-autonumbering})(.*)" select="$full-text">
									<xsl:matching-substring>
										<xsl:attribute name="numbered">
                          <xsl:value-of select="'true'" />
                        </xsl:attribute>
									</xsl:matching-substring>
								</xsl:analyze-string>
							</xsl:if>
							<xsl:apply-templates select="./*" mode="content">
								<xsl:with-param name="full-text" select="$full-text" />
							</xsl:apply-templates>
						</para>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:otherwise>
		</xsl:choose>

	</xsl:template>

</xsl:stylesheet>