<!-- 
  Template to transform XSD into PS XML

  @author Philip Rutherford
  @version 27 October 2010
-->
<xsl:stylesheet version="2.0"
				xmlns:xs="http://www.w3.org/2001/XMLSchema"
				xmlns:ps="http://www.pageseeder.com/editing/2.0"
        xmlns:psf="http://www.pageseeder.com/function"
				xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
				exclude-result-prefixes="ps xs psf">

<xsl:output method="xml" version="1.0" indent="yes" encoding="UTF-8" />

<xsl:param name="output-base" />
<xsl:param name="filename" />
<xsl:param name="split" select="'false'"/>
<xsl:param name="edit" select="'false'"/>

<xsl:variable name="schema-title" select="concat(upper-case(substring($filename,1,1)),substring($filename,2),' XSD')"/>
<xsl:variable name="ns-prefix" select="if (/xs:schema/@targetNamespace) then
    concat(/xs:schema/namespace::*[.=/xs:schema/@targetNamespace]/name(),':') else ':'"/>

<xsl:template match="/">
  <xsl:choose>
    <xsl:when test="$split='true'">
		  <xsl:text>&#xA;</xsl:text>
		  <xsl:processing-instruction name="stylesheet">format="standard" config="xsd_master" edit="<xsl:value-of select="$edit" />" title="<xsl:value-of select="$schema-title" />"</xsl:processing-instruction>
		  <root>
		  <properties>
		    <property name="document-type">xsd_master</property>
		  </properties>
		  <xsl:apply-templates select="xs:schema"/>
		  <section id="1" format="psxreflist">
		    <body>
		      <xsl:apply-templates select="xs:schema/*" mode="split"/>
		    </body>
		  </section>
		  </root>
    </xsl:when>
    <xsl:otherwise>
      <xsl:text>&#xA;</xsl:text>
      <xsl:processing-instruction name="stylesheet">format="standard" config="xsd" edit="<xsl:value-of select="$edit" />" title="<xsl:value-of select="$schema-title" />"</xsl:processing-instruction>
      <root>
      <properties>
        <property name="document-type">xsd</property>
      </properties>
      <xsl:apply-templates select="xs:schema"/>
      <xsl:apply-templates select="xs:schema/*[not(self::xs:annotation)]"/>
      </root>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template match="xs:schema">
  <section id="title">
	  <body>
	    <heading1><xsl:value-of select="$schema-title" /></heading1>
	    <para><bold>Version: </bold><xsl:value-of select="@version"/></para>
	  </body>
  </section>
  <section id="toc">
    <toc/>
  </section>
  <section id="doc">
	  <body>
	  <!-- Get first text node for backward compatibility with old format-schema -->
	  <para><xsl:copy-of select="psf:beautify-text(xs:annotation/xs:documentation/text()[1])"/></para>
	  </body>
  </section>
  <section id="code">
    <body>
    <xsl:call-template name="xsd-code"/>
    </body>
  </section>
</xsl:template>

<xsl:template match="xs:complexType" mode="split">
  <xsl:variable name="out-url" select="concat(replace($output-base, '\\', '/'),'/', $filename, '/xsd-type-', replace(@name, '\.', '-'), '.xml')" />
  <xsl:variable name="title" select="concat('Complex Type ', @name)"/>
  <xsl:result-document href="{$out-url}">
    <xsl:text>&#xA;</xsl:text>
    <xsl:processing-instruction name="stylesheet">format="standard" config="xsd_type" edit="<xsl:value-of select="$edit" />" title="<xsl:value-of select="$title" />"</xsl:processing-instruction>
    <root>
	    <properties>
	      <property name="document-type">xsd_type</property>
	    </properties>
		  <section id="title">
		    <body>
	        <heading2><xsl:value-of select="$title"/></heading2>
		    </body>
		  </section>
		  <section id="doc">
			  <body>
			  <xsl:apply-templates select="xs:annotation"/> 
        <xsl:call-template name="xsd-links"/>
        <xsl:call-template name="xsd-local-elements"/>
        <xsl:call-template name="xsd-attributes"/>
        </body>
      </section>
	    <section id="code">
	      <body>
	      <xsl:call-template name="xsd-code"/>
	      </body>
	    </section>
    </root>
  </xsl:result-document>
  <para>
  <xref frag="default" display="document" type="Replace" reverselink="true" reversetitle="" reversetype="None" href="{$filename}/xsd-type-{replace(@name, '\.', '-')}.xml"><xsl:value-of select="@name"/></xref>
  </para>
</xsl:template>

<xsl:template match="xs:group" mode="split">
  <xsl:variable name="out-url" select="concat(replace($output-base, '\\', '/'), $filename, '/xsd-group-', replace(@name, '\.', '-'), '.xml')" />
  <xsl:variable name="title" select="concat('Group ', @name)"/>
  <xsl:result-document href="{$out-url}">
    <xsl:text>&#xA;</xsl:text>
    <xsl:processing-instruction name="stylesheet">format="standard" config="xsd_group" edit="<xsl:value-of select="$edit" />" title="<xsl:value-of select="$title" />"</xsl:processing-instruction>
    <root>
      <properties>
        <property name="document-type">xsd_group</property>
      </properties>
      <section id="title">
        <body>
          <heading2><xsl:value-of select="$title"/></heading2>
        </body>
      </section>
      <section id="doc">
        <body>
        <xsl:apply-templates select="xs:annotation"/> 
        <xsl:call-template name="xsd-links"/>
        <xsl:call-template name="xsd-local-elements"/>
        <xsl:call-template name="xsd-attributes"/>
        </body>
      </section>
      <section id="code">
        <body>
        <xsl:call-template name="xsd-code"/>
        </body>
      </section>
    </root>
  </xsl:result-document>
  <para>
  <xref frag="default" display="document" type="Replace" reverselink="true" reversetitle="" reversetype="None" href="{$filename}/xsd-group-{replace(@name, '\.', '-')}.xml"><xsl:value-of select="@name"/></xref>
  </para>
</xsl:template>

<xsl:template match="xs:element"  mode="split">
  <xsl:variable name="out-url" select="concat(replace($output-base, '\\', '/'), $filename, '/xsd-element-', replace(@name, '\.', '-'), '.xml')" />
  <xsl:variable name="title" select="concat('Element ', @name)"/>
  <xsl:result-document href="{$out-url}">
    <xsl:text>&#xA;</xsl:text>
    <xsl:processing-instruction name="stylesheet">format="standard" config="xsd_element" edit="<xsl:value-of select="$edit" />" title="<xsl:value-of select="$title" />"</xsl:processing-instruction>
    <root>
      <properties>
        <property name="document-type">xsd_element</property>
      </properties>
      <section id="title">
        <body>
          <heading2><xsl:value-of select="$title"/></heading2>
        </body>
      </section>
      <section id="doc">
        <body>
        <xsl:apply-templates select="xs:annotation"/> 
        <xsl:if test="@type">
          <paraLabel name="xsd-type-ref">
	          <xsl:variable name="local-type" select="if ($ns-prefix=':' or not(contains(@type,$ns-prefix))) then @type else
	              substring-after(@type,$ns-prefix)"/>
	          <xsl:choose>
	            <xsl:when test="not(/xs:schema/xs:complexType[@name=$local-type])">
	              <xsl:value-of select="$local-type"/>
	            </xsl:when>
	            <xsl:otherwise>
	              <xref frag="default" display="manual" title="{$local-type}" type="None" reverselink="true" reversetitle="" reversetype="None" href="xsd-type-{replace($local-type, '\.', '-')}.xml"><xsl:value-of select="$local-type"/></xref>
	            </xsl:otherwise>
	          </xsl:choose>
          </paraLabel>
        </xsl:if>
        <xsl:call-template name="xsd-links"/>
        <xsl:call-template name="xsd-local-elements"/>
        <xsl:call-template name="xsd-attributes"/>
        </body>
      </section>
      <section id="code">
        <body>
        <xsl:call-template name="xsd-code"/>
        </body>
      </section>
    </root>
  </xsl:result-document>
  <para>
  <xref frag="default" display="document" type="Replace" reverselink="true" reversetitle="" reversetype="None" href="{$filename}/xsd-element-{replace(@name, '\.', '-')}.xml"><xsl:value-of select="@name"/></xref>
  </para>
</xsl:template>

<!-- ignore any other elements -->
<xsl:template match="*" mode="split">
</xsl:template>

<xsl:template match="xs:complexType">
  <section id="xsd-type-{@name}">
  <body>
  <heading2>Complex Type <xsl:value-of select="@name"/></heading2>
  <xsl:apply-templates select="xs:annotation"/> 
  <xsl:call-template name="xsd-links"/>
  <xsl:call-template name="xsd-local-elements"/>
  <xsl:call-template name="xsd-attributes"/>
  <xsl:call-template name="xsd-code"/>
  </body>
  </section>
</xsl:template>

<xsl:template match="xs:group">
  <section id="xsd-group-{@name}">
  <body>
  <heading2>Group <xsl:value-of select="@name"/></heading2>
  <xsl:apply-templates select="xs:annotation"/> 
  <xsl:call-template name="xsd-links"/>
  <xsl:call-template name="xsd-local-elements"/>
  <xsl:call-template name="xsd-attributes"/>
  <xsl:call-template name="xsd-code"/>
  </body>
  </section>
</xsl:template>

<xsl:template match="xs:element">
  <section id="xsd-element-{@name}">
  <body>
  <heading2>Element <xsl:value-of select="@name"/></heading2>
  <xsl:apply-templates select="xs:annotation"/> 
  <xsl:if test="@type">
    <paraLabel name="xsd-type-ref">
	    <xsl:variable name="local-type" select="if ($ns-prefix=':' or not(contains(@type,$ns-prefix))) then @type else
	        substring-after(@type,$ns-prefix)"/>
	    <xsl:choose>
	      <xsl:when test="not(/xs:schema/xs:complexType[@name=$local-type])">
	        <xsl:value-of select="$local-type"/>
	      </xsl:when>
	      <xsl:otherwise>
	        <link href="#xsd-type-{$local-type}"><xsl:value-of select="$local-type"/></link>
	      </xsl:otherwise>
	    </xsl:choose>
    </paraLabel>
  </xsl:if>
  <xsl:call-template name="xsd-links"/>
  <xsl:call-template name="xsd-local-elements"/>
  <xsl:call-template name="xsd-attributes"/>
  <xsl:call-template name="xsd-code"/>
  </body>
  </section>
</xsl:template>

<xsl:template match="xs:annotation">
  <xsl:choose>
    <!-- Handle old format-schema for backward compatibility -->
    <xsl:when test=".//ps:field and .//ps:content">
      <xsl:variable name="description" select="(./xs:documentation[ps:field='Description']//ps:content)[1]"/>
      <xsl:variable name="dataformat" select="(./xs:documentation[ps:field='DataFormat']//ps:content)[1]"/>
      <xsl:variable name="examples" select="(./xs:documentation[ps:field='Examples']//ps:content)[1]"/>
      <xsl:if test="$description">
        <para><xsl:copy-of select="psf:beautify-text($description)"/></para>
      </xsl:if>
      <xsl:if test="$dataformat">
        <para>Data Format: <xsl:copy-of select="psf:beautify-text($dataformat)"/></para>
      </xsl:if>
      <xsl:if test="$examples">
        <para>Examples: <xsl:copy-of select="psf:beautify-text($examples)"/></para>
      </xsl:if>
    </xsl:when>
    <xsl:when test=".//ps:field or .//ps:documentInfo or .//ps:documentinfo">
    </xsl:when>
    <xsl:otherwise>
      <para><xsl:copy-of select="psf:beautify-text(xs:documentation)"/></para>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<!-- ignore any other elements -->
<xsl:template match="*">
</xsl:template>

<xsl:template name="xsd-links">
  <xsl:if test=".//xs:group">
    <paraLabel name="xsd-group-ref">
      <xsl:for-each select=".//xs:group">
		    <xsl:variable name="local-ref" select="if ($ns-prefix=':' or not(contains(@ref,$ns-prefix))) then @ref else
		        substring-after(@ref,$ns-prefix)"/>
        <xsl:choose>
          <xsl:when test="not(/xs:schema/xs:group[@name=$local-ref])">
            <xsl:value-of select="$local-ref"/>
          </xsl:when>
          <xsl:when test="$split='true'">
            <xref frag="default" display="manual" title="{$local-ref}" type="None" reverselink="true" reversetitle="" reversetype="None" href="xsd-group-{replace($local-ref, '\.', '-')}.xml"><xsl:value-of select="$local-ref"/></xref>
          </xsl:when>
          <xsl:otherwise>
            <link href="#xsd-group-{$local-ref}"><xsl:value-of select="$local-ref"/></link>
          </xsl:otherwise>
        </xsl:choose>
        <xsl:text> </xsl:text>
      </xsl:for-each>
    </paraLabel>
  </xsl:if>
  <xsl:if test=".//xs:element[not(@name)]">
    <paraLabel name="xsd-element-ref">
      <xsl:for-each select=".//xs:element[not(@name)]">
        <xsl:variable name="local-ref" select="if ($ns-prefix=':' or not(contains(@ref,$ns-prefix))) then @ref else
          substring-after(@ref,$ns-prefix)"/>
        <xsl:choose>
          <xsl:when test="not(/xs:schema/xs:element[@name=$local-ref])">
            <xsl:value-of select="$local-ref"/>
          </xsl:when>
          <xsl:when test="$split='true'">
            <xref frag="default" display="manual" title="{$local-ref}" type="None" reverselink="true" reversetitle="" reversetype="None" href="xsd-element-{replace($local-ref, '\.', '-')}.xml"><xsl:value-of select="$local-ref"/></xref>
          </xsl:when>
          <xsl:otherwise>
            <link href="#xsd-element-{$local-ref}"><xsl:value-of select="$local-ref"/></link>
          </xsl:otherwise>
        </xsl:choose>
        <xsl:text> </xsl:text>
      </xsl:for-each>
    </paraLabel>
  </xsl:if>
</xsl:template>

<xsl:template name="xsd-local-elements">
  <xsl:if test=".//xs:element[@name]">
    <paraLabel name="xsd-element">
       <xsl:for-each select=".//xs:element[@name]">
         <para>
           <inlineLabel name="xsd-name"><xsl:value-of select="@name"/></inlineLabel>
           <xsl:text> </xsl:text>
           <xsl:choose>
              <!-- Handle old format-schema for backward compatibility -->
              <xsl:when test="xs:annotation//ps:field and xs:annotation//ps:content">
                <xsl:variable name="description" select="(xs:annotation/xs:documentation[ps:field='Description']//ps:content)[1]"/>
                <xsl:variable name="dataformat" select="(xs:annotation/xs:documentation[ps:field='DataFormat']//ps:content)[1]"/>
                <xsl:variable name="examples" select="(xs:annotation/xs:documentation[ps:field='Examples']//ps:content)[1]"/>
                <xsl:if test="$description">
                  <xsl:copy-of select="psf:beautify-text($description)"/>
                </xsl:if>
                <xsl:if test="$dataformat">
                  <br/>Data Format: <xsl:copy-of select="psf:beautify-text($dataformat)"/>
                </xsl:if>
                <xsl:if test="$examples">
                  <br/>Examples: <xsl:copy-of select="psf:beautify-text($examples)"/>
                </xsl:if>
              </xsl:when>
              <xsl:when test="xs:annotation//ps:field">
              </xsl:when>
              <xsl:otherwise>
                <xsl:copy-of select="psf:beautify-text(xs:annotation/xs:documentation)"/>
              </xsl:otherwise>
           </xsl:choose>
         </para>
       </xsl:for-each>
    </paraLabel>
  </xsl:if>
</xsl:template>

<xsl:template name="xsd-attributes">
  <xsl:if test=".//xs:attribute">
    <paraLabel name="xsd-attribute">
       <xsl:for-each select=".//xs:attribute">
	       <para>
	         <inlineLabel name="xsd-name"><xsl:value-of select="@name"/></inlineLabel>
	         <xsl:text> </xsl:text>
					 <xsl:choose>
					    <!-- Handle old format-schema for backward compatibility -->
					    <xsl:when test="xs:annotation//ps:field and xs:annotation//ps:content">
                <xsl:variable name="description" select="(xs:annotation/xs:documentation[ps:field='Description']//ps:content)[1]"/>
	              <xsl:variable name="dataformat" select="(xs:annotation/xs:documentation[ps:field='DataFormat']//ps:content)[1]"/>
	              <xsl:variable name="examples" select="(xs:annotation/xs:documentation[ps:field='Examples']//ps:content)[1]"/>
	              <xsl:if test="$description">
					        <xsl:copy-of select="psf:beautify-text($description)"/>
					      </xsl:if>
                <xsl:if test="$dataformat">
                  <br/>Data Format: <xsl:copy-of select="psf:beautify-text($dataformat)"/>
                </xsl:if>
                <xsl:if test="$examples">
                  <br/>Examples: <xsl:copy-of select="psf:beautify-text($examples)"/>
                </xsl:if>
					    </xsl:when>
					    <xsl:when test="xs:annotation//ps:field">
					    </xsl:when>
					    <xsl:otherwise>
					      <xsl:copy-of select="psf:beautify-text(xs:annotation/xs:documentation)"/>
					    </xsl:otherwise>
					 </xsl:choose>
	       </para>
       </xsl:for-each>
    </paraLabel>
  </xsl:if>
</xsl:template>

<xsl:template name="xsd-code">
  <code>
    <xsl:apply-templates select="." mode="xsd-source"/>
  </code>
</xsl:template>

<xsl:template match="*" mode="xsd-source">
  <xsl:choose>
    <xsl:when test="not(self::xs:element[@name and not(parent::xs:schema)]) and .[*|comment()|processing-instruction()|text()]">
      <xsl:text disable-output-escaping="yes"><![CDATA[&lt;]]></xsl:text>
      <xsl:value-of select="name()" /> <xsl:apply-templates select="@*" mode="xsd-source"/>
      <xsl:text disable-output-escaping="yes"><![CDATA[&gt;]]></xsl:text>
      <xsl:apply-templates select="*|comment()|processing-instruction()|text()" mode="xsd-source"/>
      <xsl:text disable-output-escaping="yes"><![CDATA[&lt;/]]></xsl:text>
      <xsl:value-of select="name()" />
      <xsl:text disable-output-escaping="yes"><![CDATA[&gt;]]></xsl:text>
    </xsl:when>
    <xsl:otherwise>
      <xsl:text disable-output-escaping="yes"><![CDATA[&lt;]]></xsl:text>
      <xsl:value-of select="name()" />
      <xsl:apply-templates select="@*" mode="xsd-source"/>
      <xsl:text disable-output-escaping="yes"><![CDATA[ /&gt;]]></xsl:text>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template match="xs:schema" mode="xsd-source">
   <xsl:text disable-output-escaping="yes"><![CDATA[&lt;]]></xsl:text>
   <xsl:value-of select="name()" />
   <xsl:apply-templates select="@*" mode="xsd-source"/>
   <xsl:for-each select="namespace::*">
     <xsl:text>&#xA;  xmlns:</xsl:text>
     <xsl:value-of select="name()" />
     <xsl:text disable-output-escaping="yes">=&quot;</xsl:text>
     <xsl:value-of select="." />
     <xsl:text disable-output-escaping="yes">&quot;</xsl:text>
   </xsl:for-each>
   <xsl:text disable-output-escaping="yes"><![CDATA[ /&gt;]]></xsl:text>
   <xsl:for-each select="xs:import|xs:include">
     <xsl:text>&#xA;</xsl:text>
     <xsl:apply-templates select="." mode="xsd-source"/>
   </xsl:for-each>
</xsl:template>

<!-- don't display schema documentation -->
<xsl:template match="xs:annotation" mode="xsd-source">
</xsl:template>

<xsl:template match="@*" mode="xsd-source">
  <xsl:text>  </xsl:text>
  <xsl:value-of select="name()" />
  <xsl:text disable-output-escaping="yes">=&quot;</xsl:text>
  <xsl:value-of select="." />
  <xsl:text disable-output-escaping="yes">&quot;</xsl:text>
</xsl:template>

<xsl:template match="text()" mode="xsd-source">
  <xsl:value-of select="." />
</xsl:template>

<xsl:template match="comment()" mode="xsd-source">
  <italic>
  <xsl:text disable-output-escaping="yes"><![CDATA[&lt;!-- ]]></xsl:text>
  <xsl:value-of select="." />
  <xsl:text disable-output-escaping="yes"><![CDATA[ --&gt;]]></xsl:text>
  </italic>
</xsl:template>

<xsl:template match="processing-instruction()" mode="xsd-source">
  <bold>
  <xsl:text disable-output-escaping="yes"><![CDATA[&lt;?]]></xsl:text>
  <xsl:value-of select="name()" /><xsl:text> </xsl:text><xsl:value-of select="." />
  <xsl:text disable-output-escaping="yes"><![CDATA[ ?&gt;]]></xsl:text>
  </bold>
</xsl:template>

<!--
  Replaces the newlines \n by <br/> tags
-->
<xsl:function name="psf:beautify-text">  
  <!-- The string process. -->
  <xsl:param name="text"/>
  <xsl:variable name="lines" select="tokenize($text, '\n')"/>
  <xsl:for-each select="$lines">
    <xsl:copy-of select="."/><br/>
  </xsl:for-each>
</xsl:function>

</xsl:stylesheet>
