<xsl:stylesheet version="2.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xs="http://www.w3.org/2001/XMLSchema"
	xmlns:ve="http://schemas.openxmlformats.org/markup-compatibility/2006"
	xmlns:o="urn:schemas-microsoft-com:office:office"
	xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships"
	xmlns:m="http://schemas.openxmlformats.org/officeDocument/2006/math"
	xmlns:v="urn:schemas-microsoft-com:vml"
	xmlns:wp="http://schemas.openxmlformats.org/drawingml/2006/wordprocessingDrawing"
	xmlns:w10="urn:schemas-microsoft-com:office:word"
	xmlns:w="http://schemas.openxmlformats.org/wordprocessingml/2006/main"
	xmlns:wne="http://schemas.microsoft.com/office/word/2006/wordml"
	xmlns:pic="http://schemas.openxmlformats.org/drawingml/2006/picture"
	xmlns:a="http://schemas.openxmlformats.org/drawingml/2006/main"
	xmlns:fn="http://www.pageseeder.com/function" xmlns:dec="java:java.net.URLDecoder">

    <!--
        @author William Liem 
        @author Tu Tak Tran 
        @author Rick Jelliffe 
        @author Christine Feng 
        @history v00001 fix external image, base directory, link, list item 
                 v00002 list, nlist, mlist, item, nitem, mitem, numbering 
                 v00003 xref 
                 v00004 fix nested inlines, paraLabel 
                        add support for inline @type and @style sup, sub, simple table border, inlineLabel, paraLabel 
                 v00005 fix support for spanning and table/para
                 v00006 fix default problem add code add mapping params for input customization 
                 v00007 added style mapping file supported mixed contents within block elements 
                        added param for applying different styles for the first and other paragraphs 
                 v00008 added three levels of mapping priority 
                 v00011 added support for page footer 
                 v00012 added support for list and nlist with auto generated list labels 
                        added support for table styling depending on the parent element 
                 v00013 added support for graphic as an inline element 
                 v00014 added support for mixed content within a list item 
        @todo numbering style. 
        @todo subdoc URI needs delimiting according to part name rules 
        @bug list itemnumbering needs to be reset? 
        @cvsid $Id: ps-to-openXML.xsl,v 1.40 2010/05/11 00:27:59 yfeng Exp $ 
        @lib ant-docxcreator.jar 
        @tag v00003 
        @style styles.xml v1.2 (pageseeder stylesheet) 
        @numbering: numbering.xml v1.2 (pageseeder numbering stylesheet)
    -->
	<xsl:output method="xml" indent="no" />
	<xsl:strip-space elements="root section body" />
    <!-- currently only support one footer for a document -->
	<xsl:include href="footer.xsl" />
	<xsl:include href="header.xsl" />
	<xsl:include href="numbering.xsl" />
	<xsl:include href="apply-styles.xsl" />
	<xsl:include href="functions.xsl" />
	<xsl:include href="tables.xsl" />

	<xsl:param name="take-style-from-type-attribute" select="'true'" />
	<xsl:param name="hardcode-bold-and-italic" select="'false'" />
	<xsl:param name="generate-continuation" select="'true'" />
	<xsl:param name="resourcefolder" />
	<xsl:param name="header-text" />
	<xsl:param name="stylemapping" />
	<xsl:param name="imagemapping" />
	<xsl:param name="templatedocument" />
	<xsl:param name="document"
		select="document(replace($templatedocument, ' ', '%20'))" />

	<xsl:param name="default-style" select="'Normal'" />
	<xsl:param name="default-table-style" select="'TableNormal'" />

	<xsl:variable name="link" select="count(.//graphic) + 3" />
	<xsl:variable name="TableWidthDxa" select="10000" />
    <!--
      Used for images path: URL decode the path but keeps the slashes.
    -->
	<xsl:function name="fn:decode-keep-slashes">
		<xsl:param name="path" />
		<xsl:value-of
			select="string-join(for $i in tokenize($path, '/') return dec:decode($i), '/')" />
	</xsl:function>

	<xsl:template match="/">
      <!-- create images map file -->
		<xsl:result-document
			href="{concat('file:///',replace($imagemapping, '\\', '/'))}" indent="yes"
			method="xml">
			<mappings>
          <!-- White space required for adding header/footer image mapping when there are no images in document -->
				<xsl:text> </xsl:text>
          <!-- for now BMP images are not supported by DocX4j so ignore them -->
				<xsl:for-each select="//graphic[not(ends-with(@src,'.bmp'))]">
					<xsl:if
						test="empty(preceding::graphic[fn:decode-keep-slashes(@src) = fn:decode-keep-slashes(current()/@src)])">
              <!-- URL decode the path -->
						<mapping path="{fn:decode-keep-slashes(@src)}" id="psimage-{generate-id()}" />
					</xsl:if>
				</xsl:for-each>
			</mappings>
		</xsl:result-document>
      <!-- ok do the rest now -->
		<xsl:apply-templates />
	</xsl:template>
  
  <!--##root##-->
	<xsl:template match="root">
		<xsl:choose>
			<xsl:when test="not(ancestor::root)">
				<xsl:result-document href="footer1.xml">
					<xsl:copy-of select="fn:createfooter()" />
				</xsl:result-document>
				<xsl:result-document href="header1.xml">
					<xsl:copy-of select="fn:createheader()" />
				</xsl:result-document>
				<xsl:call-template name="create-numbering">
					<xsl:with-param name="path" select="'.'"></xsl:with-param>
				</xsl:call-template>
				<w:document>
					<w:body>
						<xsl:apply-templates />
						<w:sectPr>
							<xsl:copy-of select="$document/w:document/w:body/w:sectPr/w:pgSz" />
							<xsl:copy-of select="$document/w:document/w:body/w:sectPr/w:type" />
							<xsl:copy-of select="$document/w:document/w:body/w:sectPr/w:pgMar" />
							<xsl:copy-of select="$document/w:document/w:body/w:sectPr/w:cols" />
						</w:sectPr>
					</w:body>
				</w:document>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates />
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
  
  <!--##properties##-->
	<xsl:template match="properties" />
  
  <!--##section##-->
	<xsl:template match="section">
		<xsl:apply-templates />
	</xsl:template>
  
  <!--##body##-->
	<xsl:template match="body">
		<xsl:apply-templates />
	</xsl:template>
    
    
    <!-- Title is imported as normal text, applying the default word heading styles -->
	<!--##title##-->
	<xsl:template match="title">
		<w:p>
			<w:pPr>
				<xsl:call-template name="apply-style" />
			</w:pPr>
			<xsl:apply-templates />
		</w:p>
	</xsl:template>

    <!-- Headings are imported as normal text, applying the default word heading styles -->
	<!--##heading##-->
	<xsl:template
		match="*[starts-with(name(),'heading') and substring-after(name(), 'heading') != '']">
        <!-- Handle headings. Style mapped from element name -->
		<w:p>
			<w:pPr>
				<xsl:call-template name="apply-style" />
			</w:pPr>
			<xsl:apply-templates />
		</w:p>
	</xsl:template>

    <!-- Headings -->
	<!--##heading##-->
	<xsl:template match="heading">
      <!-- display as bold para for now -->
		<xsl:variable name="fake">
			<para>
				<bold>
					<xsl:copy-of select="node()" />
				</bold>
			</para>
		</xsl:variable>
		<xsl:apply-templates select="$fake/para" />
	</xsl:template>
    
    
    <!--  A paralabel is imported as a normal paragraph -->
	<!--##paraLabel##-->
	<xsl:template match="paraLabel">
      <!--
        if paraLabel contains only inline elements or text, create w:p here,
       
      -->
		<xsl:variable name="id" select="concat(@name, '-', generate-id())" />
		<xsl:choose>
        <!-- when containing other block elements, including mixed content -->
        <!-- will not create w:p here -->
			<xsl:when test="fn:hasBlockElements(.)='true'">
				<xsl:apply-templates />
			</xsl:when>
			<xsl:otherwise>
          <!-- when containing only inline elements or text()-->
				<w:p>
					<w:pPr>
						<xsl:call-template name="apply-style" />
						<!--##list##-->
						<!--##nlist##-->
						<!--##item##--> 
						<xsl:if test="ancestor::item">
							<xsl:choose>
								<xsl:when test="position()=1">
									<xsl:variable name="numId"
										select="count(ancestor::*[name()='list' or name()='nlist'][last()]/
                                                            preceding::*[name()='list' or name()='nlist'][not(ancestor::list or ancestor::nlist)])+1" />
									<w:numPr>
										<w:ilvl w:val="{count(ancestor::list)+count(ancestor::nlist)-1}" />
	                    <!--
	                      new numbering.xml contains two type of list labels, w:numId 1->
	                      decimal, w:numId 2-> bullet
	                      <w:numId w:val="{if (parent::item/parent::nlist) then '1' else '2'}" />
	                    -->
										<w:numId w:val="{$numId}" />
									</w:numPr>
								</xsl:when>
								<xsl:otherwise>
									<w:ind w:left="{720*(count(ancestor::list)+count(ancestor::nlist))}" />
								</xsl:otherwise>
							</xsl:choose>
						</xsl:if>
					</w:pPr>
					<xsl:apply-templates />
				</w:p>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

  <!-- para are treated normaly -->
  <!--##para##-->
	<xsl:template match="para">
		<w:p>
			<xsl:if test="normalize-space(.)!=''">
				<w:pPr>
					<xsl:if test="string(@indent) != '' and @indent castable as xs:integer">
						<w:ind w:left="{720*number(@indent)}" />
					</xsl:if>
					<xsl:call-template name="apply-style" />
          <!--##list##-->
          <!--##nlist##-->
          <!--##item##--> 
					<xsl:if test="ancestor::item">
						<xsl:choose>
							<xsl:when test="position()=1">
								<w:numPr>
									<xsl:variable name="numId">
										<xsl:value-of
											select="count(ancestor::*[name()='list' or name()='nlist'][last()]/preceding::*[name()='list' or name()='nlist'][not(ancestor::list or ancestor::nlist)])+1"></xsl:value-of>
									</xsl:variable>
									<w:ilvl w:val="{count(ancestor::list)+count(ancestor::nlist)-1}" />
		                       <!--
		                           new numbering.xml contains two type of list labels, w:numId 1->
		                           decimal, w:numId 2-> bullet
		                       -->
		                       <!--
		                       <w:numId w:val="{if (parent::item/parent::nlist) then '1' else '2'}" />-->
									<w:numId w:val="{$numId}" />
								</w:numPr>
							</xsl:when>
							<xsl:otherwise>
								<w:ind w:left="{720*(count(ancestor::list)+count(ancestor::nlist))}" />
							</xsl:otherwise>
						</xsl:choose>
					</xsl:if>
				</w:pPr>
			</xsl:if>
			<xsl:apply-templates />
		</w:p>
	</xsl:template>
  
	<xsl:template match="text()">
      <!-- no mixed content, create a run instead -->
		<xsl:choose>
			<xsl:when test="parent::body">
				<w:p>
					<w:r>
						<w:t xml:space="preserve"><xsl:value-of select="." /></w:t>
					</w:r>
				</w:p>
			</xsl:when>
			<xsl:otherwise>
				<w:r>
					<w:rPr>
					 <!--##inlineLabel##-->
						<xsl:if test="ancestor::inlineLabel">
                <!-- if parent is an inlineLabel -->
							<xsl:choose>
								<xsl:when
									test="ancestor::inlineLabel[@name] and ($take-style-from-type-attribute = 'true')">
									<xsl:call-template name="apply-style" />
								</xsl:when>
								<xsl:otherwise>
                    <!-- otherwise, inherit style from paragraph -->
								</xsl:otherwise>
							</xsl:choose>
						</xsl:if>
						<!--##monospace##-->
						<xsl:if test="ancestor::monospace">
							<xsl:call-template name="apply-style" />
						</xsl:if>
						<!--##sup##-->
						<xsl:if test="ancestor::sup">
							<w:vertAlign w:val="superscript" />
						</xsl:if>
						<!--##sub##-->
						<xsl:if test="ancestor::sub">
							<w:vertAlign w:val="subscript" />
						</xsl:if>
						<!--##bold##-->
						<xsl:if test="ancestor::bold">
							<w:b />
						</xsl:if>
						<!--##italic##-->
						<xsl:if test="ancestor::italic ">
							<w:i />
						</xsl:if>
						<!--##underline##-->
						<xsl:if test="ancestor::underline">
							<w:u w:val="single" />
						</xsl:if>
					</w:rPr>
					<w:t xml:space="preserve"><xsl:value-of select="." /></w:t>
				</w:r>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
  
  <!--##code##-->
	<xsl:template match="code">
		<w:p>
			<w:pPr>
				<xsl:call-template name="apply-style" />
			</w:pPr>
			<w:r>
				<xsl:variable name="return-char" select="codepoints-to-string(10)" />
				<xsl:for-each select="tokenize(.,$return-char)">
					<xsl:if test="current() !='' ">
						<w:t xml:space="preserve"><xsl:value-of select="current()" /></w:t>
						<w:cr />
					</xsl:if>
				</xsl:for-each>
			</w:r>
		</w:p>
	</xsl:template>

  <!-- inlineLabels -->
  <!--##inlineLabel##-->
	<xsl:template match="inlineLabel">
		<xsl:variable name="id" select="concat(@name, '-', generate-id())" />
        <!-- <w:bookmarkStart  name="{@name}"  w:id="{$id}"/>-->
		<xsl:apply-templates />
        <!-- <w:bookmarkEnd  w:id="{$id}"/>-->
	</xsl:template>
  
  <!--##bold##-->
	<xsl:template match="bold">
		<xsl:apply-templates />
	</xsl:template>
  
  <!--##underline##-->
	<xsl:template match="underline">
		<xsl:apply-templates />
	</xsl:template>
  
  <!--##sub##-->
	<xsl:template match="sub">
		<xsl:apply-templates />
	</xsl:template>
  
  <!--##sup##-->
	<xsl:template match="sup">
		<xsl:apply-templates />
	</xsl:template>
  
  <!--##italic##-->
	<xsl:template match="italic">
		<xsl:apply-templates />
	</xsl:template>
  
  <!--##monospace##-->
	<xsl:template match="monospace">
		<xsl:apply-templates />
	</xsl:template>
  
  <!--##br##-->
	<xsl:template match="br">
		<xsl:choose>
			<xsl:when
				test="preceding-sibling::*[fn:isBlockElement(.)='true'] or following-sibling::*[fn:isBlockElement(.)='true']">
				<w:p>
					<w:r>
						<w:br />
					</w:r>
				</w:p>
			</xsl:when>
			<xsl:when test="parent::body">
				<w:p>
					<w:r>
						<w:br />
					</w:r>
				</w:p>
			</xsl:when>
			<xsl:otherwise>
				<w:r>
					<w:br />
				</w:r>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

  <!--##graphic##-->
	<xsl:template match="graphic" xmlns:file="java.io.File">
		<xsl:variable name="src" select="@src" />
		<xsl:variable name="filename">
			<xsl:choose>
				<xsl:when test="contains(tokenize(@src,'/')[last()],'.')">
					<xsl:value-of select="@src" />
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="concat(@src,'.png')" />
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<!-- 
		<xsl:variable name="fileuri"
			select="concat('file:///',$resourcefolder, fn:decode-keep-slashes($filename))"
			as="xs:string" />
		 -->
		<xsl:choose>
			<xsl:when test="ends-with($filename, '.bmp')">
				<xsl:choose>
					<xsl:when test="parent::body">
						<w:p>
							<w:r>
								<w:t>Images with BMP format are not supported.</w:t>
							</w:r>
						</w:p>
					</xsl:when>
					<xsl:otherwise>
						<w:r>
							<w:t>Images with BMP format are not supported.</w:t>
						</w:r>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:when>
		
			<xsl:otherwise>
				<xsl:choose>
					<xsl:when test="parent::body">
						<w:p>
							<xsl:call-template name="create-pict">
								<xsl:with-param name="src" select="$src" />
							</xsl:call-template>
						</w:p>
					</xsl:when>
					<xsl:otherwise>
						<xsl:call-template name="create-pict">
							<xsl:with-param name="src" select="$src" />
						</xsl:call-template>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:otherwise>
<!-- 			<xsl:otherwise> -->
<!-- 				<xsl:choose> -->
<!-- 					<xsl:when test="parent::body"> -->
<!-- 						<w:p> -->
<!-- 							<w:r> -->
<!-- 								<w:t>Referenced picture not found</w:t> -->
<!-- 							</w:r> -->
<!-- 						</w:p> -->
<!-- 					</xsl:when> -->
<!-- 					<xsl:otherwise> -->
<!-- 						<w:r> -->
<!-- 							<w:t>Referenced picture not found</w:t> -->
<!-- 						</w:r> -->
<!-- 					</xsl:otherwise> -->
<!-- 				</xsl:choose> -->
<!-- 			</xsl:otherwise> -->
		</xsl:choose>
	</xsl:template>

	<xsl:template name="create-pict">
		<xsl:param name="src" />
		<xsl:variable name="rid">
			<xsl:choose>
				<xsl:when
					test="preceding::graphic[fn:decode-keep-slashes(@src) = fn:decode-keep-slashes($src)]">
					<xsl:variable name="previous-image"
						select="(preceding::graphic[fn:decode-keep-slashes(@src) = fn:decode-keep-slashes($src)])[1]" />
					<xsl:value-of select="$previous-image/generate-id()" />
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="generate-id()" />
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="id" select="count(preceding::graphic) + 1" />
		<xsl:variable name="width">
			<xsl:choose>
				<xsl:when test="@width[contains(.,'px')]">
          <!-- 1 pixel=9525 EMU= -->
					<xsl:value-of
						select="format-number(number(substring-before(@width,'px')) * 9525, '#######')" />
				</xsl:when>
				<xsl:when test="@width">
          <!-- 1 pixel=9525 EMU= -->
					<xsl:value-of select="format-number(@width * 9525, '#######')" />
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="3048000" />
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="height">
			<xsl:choose>
				<xsl:when test="@height[contains(.,'px')]">
          <!-- 1 pixel=9525 EMU= -->
					<xsl:value-of
						select="format-number(number(substring-before(@height,'px')) * 9525, '#######')" />
				</xsl:when>
				<xsl:when test="@height">
          <!-- 1 pixel=9525 EMU= -->
					<xsl:value-of select="format-number(@height * 9525, '#######')" />
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="2032000" />
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<w:r>
			<w:drawing>
				<wp:inline distT="0" distB="0" distL="0" distR="0">
					<wp:extent cx="{$width}" cy="{$height}" />
          <!--
            currently descr is read by the docxcreator to find the target
            image in the image folder
          -->
					<wp:docPr id="{$id}" name="Picture {$rid}" descr="{fn:decode-keep-slashes($src)}" />
					<a:graphic xmlns:a="http://schemas.openxmlformats.org/drawingml/2006/main">
						<a:graphicData
							uri="http://schemas.openxmlformats.org/drawingml/2006/picture">
							<pic:pic
								xmlns:pic="http://schemas.openxmlformats.org/drawingml/2006/picture">
								<pic:nvPicPr>
                  <!-- name here is not essential  -->
									<pic:cNvPr id="0" name="whatever" />
									<pic:cNvPicPr />
								</pic:nvPicPr>
								<pic:blipFill>
                  <!-- the id in .rels, essential -->
									<a:blip r:embed="psimage-{$rid}" />
									<a:stretch>
										<a:fillRect />
									</a:stretch>
								</pic:blipFill>
								<pic:spPr>
									<a:xfrm>
										<a:off x="0" y="0" />
                    <!-- this is the size of the image frame -->
										<a:ext cx="{$width}" cy="{$height}" />
									</a:xfrm>
									<a:prstGeom prst="rect">
										<a:avLst />
									</a:prstGeom>
								</pic:spPr>
							</pic:pic>
						</a:graphicData>
					</a:graphic>
				</wp:inline>
			</w:drawing>
		</w:r>
	</xsl:template>

    <!-- consolidate xref subdoc is done during publish process -->
	<!--##xref##-->
	<xsl:template match="xref">
		<xsl:variable name="content">
			<xsl:choose>
				<xsl:when test="@title!=''">
					<xsl:value-of select="@title" />
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="@urititle" />
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<w:r>
			<w:t>
				<xsl:value-of select="$content" />
			</w:t>
		</w:r>
	</xsl:template>
  
  <!--##blockXref##-->
	<xsl:template match="blockXref">
		<xsl:choose>
			<xsl:when test="root | body">
				<xsl:apply-templates />
			</xsl:when>
			<xsl:otherwise>
				<xsl:variable name="content"
					select="if (@title != '')  then @title else @urititle" />
				<w:p>
					<w:r>
						<w:rPr>
							<w:rStyle w:val="reference" />
						</w:rPr>
						<w:t>
							<xsl:value-of select="$content" />
						</w:t>
					</w:r>
				</w:p>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

    <!-- link href="some http" -->
    <!-- will create a hyperlink in the document -->
    <!-- currently only support external link -->
    <!-- link href="some http" -->
    <!-- will create a hyperlink in the document -->
	<!--##link##-->
	<xsl:template match="link">
		<xsl:choose>
			<xsl:when test="@href[contains(.,'#')]">
				<xsl:variable name="internalref" select="substring-after(@href,'#')" />
				<w:hyperlink w:anchor="{$internalref}" w:history="1">
					<w:r>
						<w:rPr>
							<xsl:call-template name="apply-style" />

						</w:rPr>
						<w:t xml:space="preserve"><xsl:value-of select="." /></w:t>
					</w:r>
				</w:hyperlink>
			</xsl:when>
			<xsl:when test="@href[not(contains(.,'#'))]">
                <!-- only process internal link-->
				<w:r>
					<w:fldChar w:fldCharType="begin" />
				</w:r>
				<w:r>
					<w:instrText xml:space="preserve"><xsl:text> HYPERLINK "</xsl:text><xsl:value-of
						select="@href" /><xsl:text>" </xsl:text></w:instrText>
				</w:r>
				<w:r>
					<w:fldChar w:fldCharType="separate" />
				</w:r>
				<w:r>
					<w:rPr>
						<xsl:call-template name="apply-style" />
					</w:rPr>
					<w:t>
						<xsl:value-of select="." />
					</w:t>
				</w:r>
				<w:r>
					<w:fldChar w:fldCharType="end" />
				</w:r>
			</xsl:when>
			<xsl:otherwise>
                <!--anchor @name-->
				<w:bookmarkStart w:id="{count(preceding::link[@name])}"
					w:name="{@name}" />
				<w:bookmarkEnd w:id="{count(preceding::link[@name])}" />
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
  
  <!--##list##-->
  <!--##nlist##-->
	<xsl:template match="nlist | list">
        <!--
            indenting infomation is in numbering.xml and determined by list level
        -->
		<xsl:apply-templates />

	</xsl:template>
  
	<xsl:template match="mlist">
		<xsl:apply-templates />
	</xsl:template>
  
  <!--##item##-->
	<xsl:template match="item">
      <!-- level of a list item is the number of ancestor list or nlist-->
		<xsl:choose>
			<xsl:when test="text() or link or bold or italic or sup or sub or xref">
				<w:p>
					<w:pPr>
						<xsl:call-template name="apply-style" />
						<xsl:variable name="numId"
							select="count(ancestor::*[name()='list' or name()='nlist'][last()]/
                                                      preceding::*[name()='list' or name()='nlist'][not(ancestor::list or ancestor::nlist)])+1" />
						<w:numPr>
							<w:ilvl w:val="{count(ancestor::list)+count(ancestor::nlist)-1}" />
                <!--
                  new numbering.xml contains two type of list labels, w:numId 1-> decimal, w:numId 2-> bullet
                  <w:numId w:val="{if (parent::item/parent::nlist) then '1' else '2'}" />
                -->
							<w:numId w:val="{$numId}" />
						</w:numPr>
					</w:pPr>
					<xsl:apply-templates />
				</w:p>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates />
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

    <!--  If could not match any, print this error message -->
	<xsl:template match="*[ancestor::para or ancestor::mitem or ancestor::item ]"
		priority="-1">
		<w:r>
			<w:rPr>
				<w:color w:val="991111" />
			</w:rPr>
			<w:t>
				Error unprocessed element:
				<xsl:value-of select="name(.)" />
			</w:t>
		</w:r>
	</xsl:template>

</xsl:stylesheet>