<?xml version="1.0"?>
<!--
  Templates used to display the files for a project.
  @author Christophe Lauret
  @version 1 September 2010
-->
<xsl:stylesheet version="2.0" xmlns="http://www.w3.org/1999/xhtml"
                              xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                              xmlns:xs="http://www.w3.org/2001/XMLSchema"
                              xmlns:psf="http://www.pageseeder.com/function"
                              exclude-result-prefixes="xs psf">

  <xsl:output method="html" encoding="utf-8" indent="yes"/>
  
  <xsl:variable name="context-path" select="/root/header/context-path"/>

  <!-- Main template -->
  <xsl:template match="/root">
    <xsl:apply-templates select="content/*" />
  </xsl:template>
  
  <xsl:template match="siteprefix" />
  <xsl:template match="uri">
    <div class="uri-object">
      <div class="title">URI details</div>
      <div class="details">
        <span class="first">ID</span><span class="second"><xsl:value-of select="id" /></span>
        <span class="first">Title</span><span class="second"><xsl:value-of select="displaytitle" /></span>
        <span class="first">Path</span><span class="second"><xsl:value-of select="decodedpath" /></span>
      </div>
      <xsl:variable name="siteprefix" select="../siteprefix" />
      <xsl:variable name="uriFolder" select="replace(decodedpath, '/([^/]*?)$', '')" />
      <xsl:variable name="uriFilename" select="substring-after(decodedpath, concat($uriFolder, '/'))" />
      <xsl:variable name="guris" select="gropuuris/groupuri" />
      <xsl:variable name="defaultGroup" select="(.//group[concat($siteprefix, '/', replace(name, '-', '/'), '/*') = ancestor::groupuri/path])[1]" />
      <input type="hidden" name="ps-uriId"              value="{id}" />
      <input type="hidden" name="ps-uriPath"            value="{decodedpath}" />
      <input type="hidden" name="ps-uriPort"            value="{port}" />
      <input type="hidden" name="ps-uriHost"            value="{host/name}" />
      <input type="hidden" name="ps-uriScheme"          value="{scheme}" />
      <input type="hidden" name="ps-uriPathNoPrefix"    value="{substring-after(decodedpath, $siteprefix)}" />
      <input type="hidden" name="ps-uriFolder"          value="{$uriFolder}" />
      <input type="hidden" name="ps-uriFolderNoPrefix"  value="{substring-after($uriFolder, $siteprefix)}" />
      <input type="hidden" name="ps-uriFilename"        value="{$uriFilename}" />
      <xsl:if test="$defaultGroup">
        <xsl:variable name="groupPath" select="concat($siteprefix, '/', replace($defaultGroup/name, '-', '/'))" />
        <xsl:variable name="owner" select="if (string($defaultGroup/ownerdirectory) = '') then ../wo-constant/customfolder else $defaultGroup/ownerdirectory" />
        <input type="hidden" name="style-owner"         value="{$owner}" />
        <input type="hidden" name="ps-groupName"        value="{$defaultGroup/name}" />
        <input type="hidden" name="ps-uriFolderNoGroup" value="{substring-after($uriFolder, $groupPath)}" />
      </xsl:if>
      <xsl:if test="contains($uriFilename, '.')">
        <input type="hidden" name="ps-uriFilenameNoExt" value="{substring-before($uriFilename, '.')}" />
        <input type="hidden" name="ps-uriExtension"     value="{substring-after($uriFilename, '.')}" />
      </xsl:if>
      <xsl:if test="string(behavior) != ''">
        <input type="hidden" name="style-format"        value="{substring-before(behavior, '-')}" />
        <input type="hidden" name="style-config"        value="{substring-after(substring-before(behavior, '-'), '-')}" />
        <input type="hidden" name="style-css"           value="{substring-after(substring-after(behavior, '-'), '-')}" />
      </xsl:if>
      <xsl:if test="string(type) != ''">
        <input type="hidden" name="uri-type"            value="{type}" />
      </xsl:if>
    </div>
  </xsl:template>
  
  <xsl:template match="group">
    <div class="group-object">
      <div class="title">Group details</div>
      <div class="details">
        <span class="first">ID</span><span class="second"><xsl:value-of select="id" /></span>
        <span class="first">Name</span><span class="second"><xsl:value-of select="name" /></span>
        <span class="first">Owner</span><span class="second"><xsl:value-of select="ownerdirectory" /></span>
      </div>
      <xsl:variable name="siteprefix" select="../siteprefix" />
      <xsl:variable name="owner" select="if (string(ownerdirectory) = '') then ../wo-constant/customfolder else ownerdirectory" />
      <xsl:variable name="groupname" select="name" />
      <xsl:variable name="defaultGroupUri">
        <xsl:choose>
          <xsl:when test="contains(flags, 'f')">
            <xsl:sequence select="groupuris/groupuri[path = concat($siteprefix, '/', translate($groupname, '-', '/'), '/%')]" />
          </xsl:when>
          <xsl:otherwise>
            <xsl:sequence select="groupuris/groupuri[path = concat($siteprefix, '/', translate($groupname, '-', '/'), '/*')]" />
          </xsl:otherwise>
        </xsl:choose>
      </xsl:variable>
      <input type="hidden" name="ps-groupName"           value="{name}" />
      <input type="hidden" name="ps-groupId"             value="{id}" />
      <input type="hidden" name="ps-groupFolder"         value="{$siteprefix}/{translate(name, '-', '/')}" />
      <input type="hidden" name="ps-groupFolderNoPrefix" value="/{translate(name, '-', '/')}" />
      <xsl:if test="string($owner) != ''">
        <input type="hidden" name="style-owner"          value="{$owner}" />
      </xsl:if>
      <xsl:if test="$defaultGroupUri">
        <input type="hidden" name="ps-groupHost"         value="{$defaultGroupUri/groupuri/host/name}" />
        <input type="hidden" name="ps-groupPort"         value="{$defaultGroupUri/groupuri/port}" />
      </xsl:if>
    </div>
  </xsl:template>
  
  <xsl:template match="check">
    <div class="publish-job">
      <div class="status-container {publish/status}">Status: <span class="status"><xsl:value-of select="publish/status" /></span></div>
      <xsl:if test="publish/status = 'in-progress'"><div class="loading"><span>loading...</span></div></xsl:if>
      <div class="messages">
        <xsl:for-each select="publish/message[not(@logged = 'true')]">
          <div class="message {@type}">
            <xsl:choose>
              <xsl:when test="displayUriPath">
                <xsl:text>Produced file </xsl:text>
                <a href="{normalize-space(.)}" class="external">
                  <xsl:value-of select="normalize-space(.)" />
                </a>
              </xsl:when>
              <xsl:when test="string(.) = ''"><xsl:value-of select="@progress" />%</xsl:when>
              <xsl:otherwise><xsl:value-of select="." /></xsl:otherwise>
            </xsl:choose>
          </div>
        </xsl:for-each>
        <a name="last-message" id="last-message" />
      </div>
      <xsl:if test="publish/status = 'in-progress'">
        <form action="{$context-path}/admin/embed/check" class="refresh">
          <input type="hidden" name="processid"  value="{publish/@id}" />
        </form>
      </xsl:if>
    </div>
  </xsl:template>
  
  <xsl:template match="cancelled">
    <div class="cancelled">Job was cancelled</div>
  </xsl:template>
  
  <xsl:template match="error">
    <div class="error">
      <div class="title"><xsl:value-of select="title" /></div>
      <div class="message"><xsl:value-of select="message" /></div>
    </div>
  </xsl:template>
  
  <!-- ++++++++++++++++++++++++++++++++++++++ Breadcrumb Bar ++++++++++++++++++++++++++++++++++++++++++++++ -->
  <xsl:template match="uris[empty(uri)]">
    <div class="ps-bcb-breadcrumb"><div class="empty">(empty)</div></div>
  </xsl:template>
  <xsl:template match="uris[uri]">
    <div class="ps-bcb-breadcrumb">
      <a class="ps-bcb-delim" href="#">&#187;</a>
      <a class="ps-bcb-item" href="#" nodeid="" nodepath="" nodetype="uri">&#160;</a>
      <div class="ps-bcb-items">
        <ul class="ps-bcb-items-list">
          <xsl:for-each select="uri">
            <xsl:sort select="if (type = 'folder') then 1 else 2" />
            <xsl:sort select="displaytitle" />
            <li><a nodeid="{id}" nodepath="{path}" nodetype="uri" class="ps-bcb-items-list-item"
                   href="{if (type = 'folder') then concat($context-path, '/admin/embed/browse/uris?uri=', id) else '#'}">
                  <xsl:value-of select="displaytitle" />
            </a></li>
          </xsl:for-each>
        </ul>
      </div>
    </div>
  </xsl:template>
  
</xsl:stylesheet>
